@@ -1,7 +1,7 @@
 /*
   clean.c -- clean up misuse of presentation markup
 
-  (c) 1998 (W3C) MIT, INRIA, Keio University
+  (c) 1998-2000 (W3C) MIT, INRIA, Keio University
   See tidy.c for the copyright notice.
 
   Filters from other formats such as Microsoft Word
@@ -47,7 +47,7 @@
 
 Node *CleanNode(Lexer *lexer, Node *node);
 
-void FreeStyleProps(StyleProp *props)
+static void FreeStyleProps(StyleProp *props)
 {
     StyleProp *next;
 
@@ -61,7 +61,7 @@ void FreeStyleProps(StyleProp *props)
     }
 }
 
-StyleProp *InsertProperty(StyleProp *props, char *name, char *value)
+static StyleProp *InsertProperty(StyleProp *props, char *name, char *value)
 {
     StyleProp *first, *prev, *prop;
     int cmp;
@@ -75,8 +75,8 @@ StyleProp *InsertProperty(StyleProp *props, char *name, char *value)
 
         if (cmp == 0)
         {
-            /* need to merge values */
-            break;
+            /* this property is already defined, ignore new value */
+            return first;
         }
 
         if (cmp > 0)
@@ -120,9 +120,9 @@ StyleProp *InsertProperty(StyleProp *props, char *name, char *value)
  Some systems don't allow you to null literal strings,
  so to avoid this, a copy is made first.
 */
-StyleProp *CreateProps(StyleProp *prop, char *style)
+static StyleProp *CreateProps(StyleProp *prop, char *style)
 {
-    char *name, *value, *name_end, *value_end;
+    char *name, *value = null, *name_end, *value_end;
     Bool more;
 
     style = wstrdup(style);
@@ -186,7 +186,7 @@ StyleProp *CreateProps(StyleProp *prop, char *style)
     return prop;
 }
 
-char *CreatePropString(StyleProp *props)
+static char *CreatePropString(StyleProp *props)
 {
     char *style, *p, *s;
     int len;
@@ -229,7 +229,7 @@ char *CreatePropString(StyleProp *props)
 /*
   create string with merged properties
 */
-char *AddProperty(char *style, char *property)
+static char *AddProperty(char *style, char *property)
 {
     StyleProp *prop;
 
@@ -255,7 +255,7 @@ void FreeStyles(Lexer *lexer)
     }
 }
 
-char *GensymClass(char *tag)
+static char *GensymClass(char *tag)
 {
     static int n = 1;
     char buf[128];
@@ -264,7 +264,7 @@ char *GensymClass(char *tag)
     return wstrdup(buf);
 }
 
-char *FindStyle(Lexer *lexer, char *tag, char *properties)
+static char *FindStyle(Lexer *lexer, char *tag, char *properties)
 {
     Style *style;
 
@@ -284,6 +284,33 @@ char *FindStyle(Lexer *lexer, char *tag, char *properties)
     return style->tag_class;
 }
 
+/*
+ Add class="foo" to node
+*/
+void AddClass(Node *node, char *classname)
+{
+    AttVal *classattr = GetAttrByName(node, "class");
+
+    /*
+     if there already is a class attribute
+     then append class name after a space
+    */
+    if (classattr)
+    {
+        int len = wstrlen(classattr->value) +
+                            wstrlen(classname) + 2;
+        char *s = (char *)malloc(len *sizeof(char));
+        wstrcpy(s, classattr->value);
+        wstrcat(s, " ");
+        wstrcat(s, classname);
+        MemFree(classattr->value);
+        classattr->value = s;
+    }
+    else /* create new class attribute */
+        AddAttribute(node, "class", classname);
+}
+
+
 /*
  Find style attribute in node, and replace it
  by corresponding class attribute. Search for
@@ -292,37 +319,186 @@ char *FindStyle(Lexer *lexer, char *tag, char *properties)
 
  Assumes that node doesn't have a class attribute
 */
-void Style2Rule(Lexer *lexer, Node *node)
+static void Style2Rule(Lexer *lexer, Node *node)
 {
-    AttVal *av;
+    AttVal *styleattr, *classattr;
     char *classname;
 
-    for (av = node->attributes; av; av = av->next)
-    {
-        if (wstrcmp(av->attribute, "style") == 0)
-            break;
-    }
+    styleattr = GetAttrByName(node, "style");
 
-    /* if style attribute already exists then append property */
-
-    if (av)
+    if (styleattr)
     {
-        classname = FindStyle(lexer, node->element, av->value);
-        MemFree(av->attribute);
-        MemFree(av->value);
-        av->attribute = wstrdup("class");
-        av->value = wstrdup(classname);
+        classname = FindStyle(lexer, node->element, styleattr->value);
+        classattr = GetAttrByName(node, "class");
+
+        /*
+         if there already is a class attribute
+         then append class name after a space
+        */
+        if (classattr)
+        {
+            int len = wstrlen(classattr->value) +
+                                wstrlen(classname) + 2;
+            char *s = (char *)malloc(len *sizeof(char));
+            wstrcpy(s, classattr->value);
+            wstrcat(s, " ");
+            wstrcat(s, classname);
+            MemFree(classattr->value);
+            classattr->value = s;
+            RemoveAttribute(node, styleattr);
+        }
+        else /* reuse style attribute for class attribute */
+        {
+            MemFree(styleattr->attribute);
+            MemFree(styleattr->value);
+            styleattr->attribute = wstrdup("class");
+            styleattr->value = wstrdup(classname);
+        }
     }
 }
 
-/* create style element using rules from dictionary */
-void CreateStyleElement(Lexer *lexer, Node *doc)
+static void AddColorRule(Lexer *lexer, char *selector, char *color)
 {
-    Node *node, *html, *head;
+    if (color)
+    {
+        AddStringLiteral(lexer, selector);
+        AddStringLiteral(lexer, " { color: ");
+        AddStringLiteral(lexer, color);
+        AddStringLiteral(lexer, " }\n");
+    }
+}
+
+/*
+ move presentation attribs from body to style element
+
+ background="foo" ->  body { background-image: url(foo) }
+ bgcolor="foo"    ->  body { background-color: foo }
+ text="foo"       ->  body { color: foo }
+ link="foo"       ->  :link { color: foo }
+ vlink="foo"      ->  :visited { color: foo }
+ alink="foo"      ->  :active { color: foo }
+*/
+static void CleanBodyAttrs(Lexer *lexer, Node *body)
+{
+    AttVal *attr;
+    char *bgurl = null;
+    char *bgcolor = null;
+    char *color = null;
+    
+    attr = GetAttrByName(body, "background");
+
+    if (attr)
+    {
+        bgurl = attr->value;
+        attr->value = null;
+        RemoveAttribute(body, attr);
+    }
+
+    attr = GetAttrByName(body, "bgcolor");
+
+    if (attr)
+    {
+        bgcolor = attr->value;
+        attr->value = null;
+        RemoveAttribute(body, attr);
+    }
+
+    attr = GetAttrByName(body, "text");
+
+    if (attr)
+    {
+        color = attr->value;
+        attr->value = null;
+        RemoveAttribute(body, attr);
+    }
+
+    if (bgurl || bgcolor || color)
+    {
+        AddStringLiteral(lexer, " body {\n");
+
+        if (bgurl)
+        {
+            AddStringLiteral(lexer, "  background-image: url(");
+            AddStringLiteral(lexer, bgurl);
+            AddStringLiteral(lexer, ");\n");
+            MemFree(bgurl);
+        }
+
+        if (bgcolor)
+        {
+            AddStringLiteral(lexer, "  background-color: ");
+            AddStringLiteral(lexer, bgcolor);
+            AddStringLiteral(lexer, ";\n");
+            MemFree(bgcolor);
+        }
+
+        if (color)
+        {
+            AddStringLiteral(lexer, "  color: ");
+            AddStringLiteral(lexer, color);
+            AddStringLiteral(lexer, ";\n");
+            MemFree(color);
+        }
+
+        AddStringLiteral(lexer, " }\n");
+    }
+
+    attr = GetAttrByName(body, "link");
+
+    if (attr)
+    {
+        AddColorRule(lexer, " :link", attr->value);
+        RemoveAttribute(body, attr);
+    }
+
+    attr = GetAttrByName(body, "vlink");
+
+    if (attr)
+    {
+        AddColorRule(lexer, " :visited", attr->value);
+        RemoveAttribute(body, attr);
+    }
+
+    attr = GetAttrByName(body, "alink");
+
+    if (attr)
+    {
+        AddColorRule(lexer, " :active", attr->value);
+        RemoveAttribute(body, attr);
+    }
+}
+
+static Bool NiceBody(Lexer *lexer, Node *doc)
+{
+    Node *body = FindBody(doc);
+
+    if (body)
+    {
+        if (
+            GetAttrByName(body, "background") ||
+            GetAttrByName(body, "bgcolor") ||
+            GetAttrByName(body, "text") ||
+            GetAttrByName(body, "link") ||
+            GetAttrByName(body, "vlink") ||
+            GetAttrByName(body, "alink")
+           )
+        {
+            lexer->badLayout |= USING_BODY;
+            return no;
+        }
+    }
+
+    return yes;
+}
+
+/* create style element using rules from dictionary */
+static void CreateStyleElement(Lexer *lexer, Node *doc)
+{
+    Node *node, *head, *body;
     Style *style;
     AttVal *av;
 
-    if (lexer->styles == null)
+    if (lexer->styles == null && NiceBody(lexer, doc))
         return;
 
     node = NewNode();
@@ -332,16 +508,20 @@ void CreateStyleElement(Lexer *lexer, Node *doc)
     FindTag(node);
 
     /* insert type attribute */
-    av = (AttVal *)MemAlloc(sizeof(AttVal));
-    av->next = null; 
+    av = NewAttribute();
     av->attribute = wstrdup("type");
     av->value = wstrdup("text/css");
     av->delim = '"';
     av->dict = FindAttribute(av);
     node->attributes = av;
 
+    body = FindBody(doc);
+
     lexer->txtstart = lexer->lexsize;
 
+    if (body)
+        CleanBodyAttrs(lexer, body);
+
     for (style = lexer->styles; style; style = style->next)
     {
         AddCharToLexer(lexer, ' ');
@@ -357,7 +537,7 @@ void CreateStyleElement(Lexer *lexer, Node *doc)
 
     lexer->txtend = lexer->lexsize;
 
-    InsertNode(node, TextToken(lexer));
+    InsertNodeAtEnd(node, TextToken(lexer));
 
     /*
      now insert style element into document head
@@ -366,20 +546,18 @@ void CreateStyleElement(Lexer *lexer, Node *doc)
      the head node should be first child of html node
     */
 
-    for (html = doc->content; html; html = html->next)
-    {
-        if (html->tag == tag_html)
-        {
-            head = html->content;
-            InsertNode(head, node);
-            break;
-        }
-    }
+    head = FindHead(doc);
+    
+    if (head)
+        InsertNodeAtEnd(head, node);
 }
 
+
 /* ensure bidirectional links are consistent */
-void FixNodeLinks(Node *node)
+static void FixNodeLinks(Node *node)
 {
+    Node *child;
+
     if (node->prev)
         node->prev->next = node;
     else
@@ -389,13 +567,16 @@ void FixNodeLinks(Node *node)
         node->next->prev = node;
     else
         node->parent->last = node;
+
+    for (child = node->content; child; child = child->next)
+        child->parent = node;
 }
 
 /*
  used to strip child of node when
  the node has one and only one child
 */
-void StripOnlyChild(Node *node)
+static void StripOnlyChild(Node *node)
 {
     Node *child;
 
@@ -410,7 +591,7 @@ void StripOnlyChild(Node *node)
 }
 
 /* used to strip font start and end tags */
-void DiscardContainer(Node *element, Node **pnode)
+static void DiscardContainer(Node *element, Node **pnode)
 {
     Node *node, *parent = element->parent;
 
@@ -462,7 +643,7 @@ void DiscardContainer(Node *element, Node **pnode)
  Add style property to element, creating style
  attribute as needed and adding ; delimiter
 */
-void AddStyleProperty(Node *node, char *property)
+static void AddStyleProperty(Node *node, char *property)
 {
     AttVal *av;
 
@@ -484,7 +665,7 @@ void AddStyleProperty(Node *node, char *property)
     }
     else /* else create new style attribute */
     {
-        av = (AttVal *)MemAlloc(sizeof(AttVal));
+        av = NewAttribute();
         av->attribute = wstrdup("style");
         av->value = wstrdup(property);
         av->delim = '"';
@@ -503,7 +684,7 @@ void AddStyleProperty(Node *node, char *property)
   into the list in order, merging values for
   the same property name.
 */
-char *MergeProperties(char *s1, char *s2)
+static char *MergeProperties(char *s1, char *s2)
 {
     char *s;
     StyleProp *prop;
@@ -515,7 +696,7 @@ char *MergeProperties(char *s1, char *s2)
     return s;
 }
 
-void MergeStyles(Node *node, Node *child)
+static void MergeStyles(Node *node, Node *child)
 {
     AttVal *av;
     char *s1, *s2, *style;
@@ -549,7 +730,7 @@ void MergeStyles(Node *node, Node *child)
     }
     else if (s2)  /* copy style of child */
     {
-        av = (AttVal *)MemAlloc(sizeof(AttVal));
+        av = NewAttribute();
         av->attribute = wstrdup("style");
         av->value = wstrdup(s2);
         av->delim = '"';
@@ -559,13 +740,22 @@ void MergeStyles(Node *node, Node *child)
     }
 }
 
-char *FontSize2Name(char *size)
+static char *FontSize2Name(char *size)
 {
-    char *sizes[7] =
+#if 0
+    static char *sizes[7] =
       {
         "50%", "60%", "80%", null,
         "120%", "150%", "200%"
       };
+#else
+    static char *sizes[7] =
+      {
+        "60%", "70%", "80%", null,
+        "120%", "150%", "200%"
+      };
+#endif
+    static char buf[16];
 
     if ('0' <= size[0] && size[0] <= '6')
     {
@@ -574,12 +764,42 @@ char *FontSize2Name(char *size)
     }
 
     if (size[0] == '-')
-        return "smaller";
+    {
+        if ('0' <= size[1] && size[1] <= '6')
+        {
+            int n = size[1] - '0';
+            double x;
 
-    return "larger";
+            for (x = 1; n > 0; --n)
+                x *= 0.8;
+
+            x *= 100;
+            sprintf(buf, "%d%%", (int)(x));
+
+            return buf;
+        }
+
+        return "smaller"; /*"70%"; */
+    }
+
+    if ('0' <= size[1] && size[1] <= '6')
+    {
+        int n = size[1] - '0';
+        double x;
+
+        for (x = 1; n > 0; --n)
+            x *= 1.2;
+
+        x *= 100;
+        sprintf(buf, "%d%%", (int)(x));
+
+        return buf;
+    }
+
+    return "larger"; /* "140%" */
 }
 
-void AddFontFace(Node *node, char *face)
+static void AddFontFace(Node *node, char *face)
 {
     char buf[1024];
 
@@ -587,7 +807,7 @@ void AddFontFace(Node *node, char *face)
     AddStyleProperty(node, buf);
 }
 
-void AddFontSize(Node *node, char *size)
+static void AddFontSize(Node *node, char *size)
 {
     char *value, buf[1024];
 
@@ -624,7 +844,7 @@ void AddFontSize(Node *node, char *size)
     }
 }
 
-void AddFontColor(Node *node, char *color)
+static void AddFontColor(Node *node, char *color)
 {
     char buf[1024];
 
@@ -632,7 +852,7 @@ void AddFontColor(Node *node, char *color)
     AddStyleProperty(node, buf);
 }
 
-void AddAlign(Node *node, char *align)
+static void AddAlign(Node *node, char *align)
 {
     char buf[1024], *p, *q;
 
@@ -646,7 +866,7 @@ void AddAlign(Node *node, char *align)
  add style properties to node corresponding to
  the font face, size and color attributes
 */
-void AddFontStyles(Node *node, AttVal *av)
+static void AddFontStyles(Node *node, AttVal *av)
 {
     while (av)
     {
@@ -665,7 +885,7 @@ void AddFontStyles(Node *node, AttVal *av)
     Symptom: <p align=center>
     Action: <p style="text-align: center">
 */
-void TextAlign(Lexer *lexer, Node *node)
+static void TextAlign(Lexer *lexer, Node *node)
 {
     AttVal *av, *prev;
 
@@ -678,7 +898,7 @@ void TextAlign(Lexer *lexer, Node *node)
             if (prev)
                 prev->next = av->next;
             else
-                node->attributes = null;
+                node->attributes = av->next;
 
             MemFree(av->attribute);
 
@@ -706,7 +926,7 @@ void TextAlign(Lexer *lexer, Node *node)
     Action: coerce <dir> <li> to <div> with indent.
 */
 
-Bool Dir2Div(Lexer *lexer, Node *node, Node **pnode)
+static Bool Dir2Div(Lexer *lexer, Node *node, Node **pnode)
 {
     Node *child;
 
@@ -772,40 +992,12 @@ Bool Dir2Div(Lexer *lexer, Node *node, Node **pnode)
     return no;
 }
 
-/*
-    Symptom: implicit blockquote due to <ul>some text</ul>
-    Action: replace by <div> with indent style
-*/
-Bool Blockquote2Div(Lexer *lexer, Node *node, Node **pnode)
-{
-    Node *child;
-
-    if (node->tag == tag_blockquote && node->implicit)
-    {
-        child = node->content;
-
-        if (child == null)
-            return no;
-
-        /* coerce blockquote to div */
-
-        node->tag = tag_div;
-        MemFree(node->element);
-        node->element = wstrdup("div");
-        AddStyleProperty(node, "margin-left: 2em");
-        return yes;
-    }
-
-    return no;
-}
-
-
 /*
     Symptom: <center>
     Action: replace <center> by <div style="text-align: center">
 */
 
-Bool Center2Div(Lexer *lexer, Node *node, Node **pnode)
+static Bool Center2Div(Lexer *lexer, Node *node, Node **pnode)
 {
     if (node->tag == tag_center)
     {
@@ -826,7 +1018,7 @@ Bool Center2Div(Lexer *lexer, Node *node, Node **pnode)
                 last->next = node;
                 node->prev = last;
 
-                if (parent->last == last) /* RSE: was "=", but I think it was a bug! Really? */
+                if (parent->last == last)
                     parent->last = node;
 
                 node->parent = parent;
@@ -872,7 +1064,7 @@ Bool Center2Div(Lexer *lexer, Node *node, Node **pnode)
   This is useful after nested <dir>s used by Word
   for indenting have been converted to <div>s
 */
-Bool MergeDivs(Lexer *lexer, Node *node, Node **pnode)
+static Bool MergeDivs(Lexer *lexer, Node *node, Node **pnode)
 {
     Node *child;
 
@@ -900,7 +1092,7 @@ Bool MergeDivs(Lexer *lexer, Node *node, Node **pnode)
     Action: discard outer list
 */
 
-Bool NestedList(Lexer *lexer, Node *node, Node **pnode)
+static Bool NestedList(Lexer *lexer, Node *node, Node **pnode)
 {
     Node *child, *list;
 
@@ -973,11 +1165,11 @@ Bool NestedList(Lexer *lexer, Node *node, Node **pnode)
 }
 
 /*
-    Symptom: the only child of a block-level element is a
-    presentation element such as B, I or FONT
+  Symptom: the only child of a block-level element is a
+  presentation element such as B, I or FONT
 
-    Action: add style "font-weight: bold" to the block and
-    strip the <b> element, leaving its children.
+  Action: add style "font-weight: bold" to the block and
+  strip the <b> element, leaving its children.
 
   example:
 
@@ -990,58 +1182,70 @@ Bool NestedList(Lexer *lexer, Node *node, Node **pnode)
       <p style="font-weight: bold; font-family: Arial; font-size: 6">
         Draft Recommended Practice
       </p>
+
+  This code also replaces the align attribute by a style attribute.
+  However, to avoid CSS problems with Navigator 4, this isn't done
+  for the elements: caption, tr and table
 */
-Bool BlockStyle(Lexer *lexer, Node *node, Node **pnode)
+static Bool BlockStyle(Lexer *lexer, Node *node, Node **pnode)
 {
     Node *child;
 
     if (node->tag->model & (CM_BLOCK | CM_LIST | CM_DEFLIST | CM_TABLE))
     {
-        /* check for align attribute */
-        if (node->tag != tag_caption)
-            TextAlign(lexer, node);
-
-        child = node->content;
-
-        if (child == null)
-            return no;
-
-        /* check child has no peers */
-
-        if (child->next)
-            return no;
-
-        if (child->tag == tag_b)
+        if (node->tag != tag_table
+                && node->tag != tag_tr
+                && node->tag != tag_li)
         {
-            AddStyleProperty(node, "font-weight: bold");
-            StripOnlyChild(node);
-            return yes;
-        }
+            /* check for align attribute */
+            if (node->tag != tag_caption)
+                TextAlign(lexer, node);
 
-        if (child->tag == tag_i)
-        {
-            AddStyleProperty(node, "font-style: italic");
-            StripOnlyChild(node);
-            return yes;
-        }
+            child = node->content;
 
-        if (child->tag == tag_font)
-        {
-            AddFontStyles(node, child->attributes);
-            StripOnlyChild(node);
-            return yes;
+            if (child == null)
+                return no;
+
+            /* check child has no peers */
+
+            if (child->next)
+                return no;
+
+            if (child->tag == tag_b)
+            {
+                MergeStyles(node, child);
+                AddStyleProperty(node, "font-weight: bold");
+                StripOnlyChild(node);
+                return yes;
+            }
+
+            if (child->tag == tag_i)
+            {
+                MergeStyles(node, child);
+                AddStyleProperty(node, "font-style: italic");
+                StripOnlyChild(node);
+                return yes;
+            }
+
+            if (child->tag == tag_font)
+            {
+                MergeStyles(node, child);
+                AddFontStyles(node, child->attributes);
+                StripOnlyChild(node);
+                return yes;
+            }
         }
     }
 
     return no;
 }
 
-/* the only child of an inline element such as em */
-Bool InlineStyle(Lexer *lexer, Node *node, Node **pnode)
+/* the only child of table cell or an inline element such as em */
+static Bool InlineStyle(Lexer *lexer, Node *node, Node **pnode)
 {
     Node *child;
 
-    if (node->tag != tag_font && (node->tag->model & CM_INLINE))
+    if (node->tag != tag_font && (node->tag->model & (CM_INLINE|CM_ROW)))
     {
         child = node->content;
 
@@ -1053,15 +1257,17 @@ Bool InlineStyle(Lexer *lexer, Node *node, Node **pnode)
         if (child->next)
             return no;
 
-        if (child->tag == tag_b)
+        if (child->tag == tag_b && LogicalEmphasis)
         {
+            MergeStyles(node, child);
             AddStyleProperty(node, "font-weight: bold");
             StripOnlyChild(node);
             return yes;
         }
 
-        if (child->tag == tag_i)
+        if (child->tag == tag_i && LogicalEmphasis)
         {
+            MergeStyles(node, child);
             AddStyleProperty(node, "font-style: italic");
             StripOnlyChild(node);
             return yes;
@@ -1069,6 +1275,7 @@ Bool InlineStyle(Lexer *lexer, Node *node, Node **pnode)
 
         if (child->tag == tag_font)
         {
+            MergeStyles(node, child);
             AddFontStyles(node, child->attributes);
             StripOnlyChild(node);
             return yes;
@@ -1083,7 +1290,7 @@ Bool InlineStyle(Lexer *lexer, Node *node, Node **pnode)
   the font element's attributes and replacing them
   by a single style attribute.
 */
-Bool Font2Span(Lexer *lexer, Node *node, Node **pnode)
+static Bool Font2Span(Lexer *lexer, Node *node, Node **pnode)
 {
     AttVal *av, *style, *next;
 
@@ -1102,6 +1309,7 @@ Bool Font2Span(Lexer *lexer, Node *node, Node **pnode)
 
         AddFontStyles(node, node->attributes);
 
+        /* extract style attribute and free the rest */
         av = node->attributes;
         style = null;
 
@@ -1139,7 +1347,7 @@ Bool Font2Span(Lexer *lexer, Node *node, Node **pnode)
     return no;
 }
 
-Bool IsElement(Node *node)
+static Bool IsElement(Node *node)
 {
     return (node->type == StartTag || node->type == StartEndTag ? yes : no);
 }
@@ -1156,9 +1364,6 @@ Node *CleanNode(Lexer *lexer, Node *node)
         if (Dir2Div(lexer, node, &next))
             continue;
 
-        if (Blockquote2Div(lexer, node, &next))
-            continue;
-
         if (NestedList(lexer, node, &next))
             continue;
 
@@ -1183,7 +1388,7 @@ Node *CleanNode(Lexer *lexer, Node *node)
     return next;
 }
 
-Node *CreateStyleProperties(Lexer *lexer, Node *node)
+static Node *CreateStyleProperties(Lexer *lexer, Node *node)
 {
     Node *child;
 
@@ -1198,7 +1403,7 @@ Node *CreateStyleProperties(Lexer *lexer, Node *node)
     return CleanNode(lexer, node);
 }
 
-void DefineStyleRules(Lexer *lexer, Node *node)
+static void DefineStyleRules(Lexer *lexer, Node *node)
 {
     Node *child;
 
@@ -1218,13 +1423,38 @@ void CleanTree(Lexer *lexer, Node *doc)
 {
     doc = CreateStyleProperties(lexer, doc);
 
-    if (!DropFontTags)
+    if (MakeClean)
     {
         DefineStyleRules(lexer, doc);
         CreateStyleElement(lexer, doc);
     }
 }
 
+/* simplifies <b><b> ... </b> ...</b> etc. */
+void NestedEmphasis(Node *node)
+{
+    Node *next;
+
+    while (node)
+    {
+        next = node->next;
+
+        if ((node->tag == tag_b || node->tag == tag_i)
+            && node->parent && node->parent->tag == node->tag)
+        {
+            /* strip redundant inner element */
+            DiscardContainer(node, &next);
+            node = next;
+            continue;
+        }
+
+        if (node->content)
+            NestedEmphasis(node->content);
+
+        node = next;
+    }
+}
+
 /* replace i by em and b by strong */
 void EmFromI(Node *node)
 {
@@ -1250,8 +1480,84 @@ void EmFromI(Node *node)
     }
 }
 
+static Bool HasOneChild(Node *node)
+{
+    return (node->content && node->content->next == null);
+}
+
+/*
+ Some people use dir or ul without an li
+ to indent the content. The pattern to
+ look for is a list with a single implicit
+ li. This is recursively replaced by an
+ implicit blockquote.
+*/
+void List2BQ(Node *node)
+{
+    while (node)
+    {
+        if (node->content)
+            List2BQ(node->content);
+
+        if (node->tag && node->tag->parser == ParseList &&
+            HasOneChild(node) && node->content->implicit)
+        {
+            StripOnlyChild(node);
+            MemFree(node->element);
+            node->element = wstrdup(tag_blockquote->name);
+            node->tag = tag_blockquote;
+            node->implicit = yes;
+        }
+
+        node = node->next;
+    }
+}
+
+static char indent_buf[32];
+
+/*
+ Replace implicit blockquote by div with an indent
+ taking care to reduce nested blockquotes to a single
+ div with the indent set to match the nesting depth
+*/
+void BQ2Div(Node *node)
+{
+    int indent;
+
+    while (node)
+    {
+        if (node->tag == tag_blockquote && node->implicit)
+        {
+            indent = 1;
+
+            while(HasOneChild(node) &&
+                  node->content->tag == tag_blockquote &&
+                  node->implicit)
+            {
+                ++indent;
+                StripOnlyChild(node);
+            }
+
+            if (node->content)
+                BQ2Div(node->content);
+
+            sprintf(indent_buf, "margin-left: %dem", 2*indent);
+
+            MemFree(node->element);
+            node->element = wstrdup(tag_div->name);
+            node->tag = tag_div;
+            AddAttribute(node, "style", indent_buf);
+        }
+        else if (node->content)
+            BQ2Div(node->content);
+
+
+        node = node->next;
+    }
+}
+
 /* node is <![if ...]> prune up to <![endif]> */
-Node *PruneSection(Lexer *lexer, Node *node)
+static Node *PruneSection(Lexer *lexer, Node *node)
 {
     for (;;)
     {
@@ -1305,7 +1611,7 @@ void DropSections(Lexer *lexer, Node *node)
     }
 }
 
-void PurgeAttributes(Node *node)
+static void PurgeAttributes(Node *node)
 {
     AttVal *attr = node->attributes, *next, *prev = null;
 
@@ -1313,11 +1619,21 @@ void PurgeAttributes(Node *node)
     {
         next = attr->next;
 
-        if (wstrcmp(attr->attribute, "class") == 0 ||
+        /* special check for class="Code" denoting pre text */
+        if (wstrcmp(attr->attribute, "class") == 0 &&
+            wstrcmp(attr->value, "Code") == 0)
+        {
+            prev = attr;
+        }
+        else if (wstrcmp(attr->attribute, "class") == 0 ||
             wstrcmp(attr->attribute, "style") == 0 ||
             wstrcmp(attr->attribute, "lang") == 0 ||
-            (wstrcmp(attr->attribute, "width") == 0 &&
-                (node->tag == tag_td || node->tag == tag_th)))
+            wstrncmp(attr->attribute, "x:", 2) == 0 ||
+            ((wstrcmp(attr->attribute, "heigy") == 0 ||
+              wstrcmp(attr->attribute, "width") == 0) &&
+                (node->tag == tag_td ||
+                 node->tag == tag_tr ||
+                 node->tag == tag_th)))
         {
             if (prev)
                 prev->next = next;
@@ -1334,9 +1650,9 @@ void PurgeAttributes(Node *node)
 }
 
 /* Word2000 uses span excessively, so we strip span out */
-Node *StripSpan(Node *span)
+static Node *StripSpan(Lexer *lexer, Node *span)
 {
-    Node *node, *prev, *content;
+    Node *node, *prev = null, *content;
 
     /*
      deal with span elements that have content
@@ -1344,7 +1660,7 @@ Node *StripSpan(Node *span)
      after having processed it
     */
 
-    CleanWord2000(span->content);
+    CleanWord2000(lexer, span->content);
     content = span->content;
 
     if (span->prev)
@@ -1376,15 +1692,50 @@ Node *StripSpan(Node *span)
     return node;
 }
 
+/* map non-breaking spaces to regular spaces */
+static void NormalizeSpaces(Lexer *lexer, Node *node)
+{
+    while (node)
+    {
+        if (node->content)
+            NormalizeSpaces(lexer, node->content);
+
+        if (node->type == TextNode)
+        {
+            unsigned int i, c;
+            char *p = lexer->lexbuf + node->start;
+
+            for (i = node->start; i < node->end; ++i)
+            {
+                c = (unsigned char)lexer->lexbuf[i];
+
+                /* look for UTF-8 multibyte character */
+                if (c > 0x7F)
+                    i += GetUTF8((unsigned char *)(lexer->lexbuf + i), &c);
+
+                if (c == 160)
+                    c = ' ';
+
+                p = PutUTF8(p, c);
+            }
+        }
+
+        node = node->next;
+    }
+}
+
 /*
- This is a major clean up to strip out all the crud you get when
- you save as web page from Word 2000. It doesn't yet know what to
- do with VML tags, but these will appear as errors unless you
- declare them as new tags such as o:p which needs to be declared
+ This is a major clean up to strip out all the extra stuff you get
+ when you save as web page from Word 2000. It doesn't yet know what
+ to do with VML tags, but these will appear as errors unless you
+ declare them as new tags, such as o:p which needs to be declared
  as inline.
 */
-void CleanWord2000(Node *node)
+void CleanWord2000(Lexer *lexer, Node *node)
 {
+    /* used to a list from a sequence of bulletted p's */
+    Node *list = null;
+
     while (node)
     {
         /* discard Word's style verbiage */
@@ -1397,7 +1748,7 @@ void CleanWord2000(Node *node)
         /* strip out all span tags Word scatters so liberally! */
         if (node->tag == tag_span)
         {
-            node = StripSpan(node);
+            node = StripSpan(lexer, node);
             continue;
         }
 
@@ -1422,13 +1773,6 @@ void CleanWord2000(Node *node)
             }
         }
 
-        /* strip out style and class attributes */
-        if (node->type == StartTag || node->type == StartEndTag)
-            PurgeAttributes(node);
-
-        if (node->content)
-            CleanWord2000(node->content);
-
         /* discard empty paragraphs */
         if (node->content == null && node->tag == tag_p)
         {
@@ -1436,6 +1780,70 @@ void CleanWord2000(Node *node)
             continue;
         }
 
+        if (node->tag == tag_p)
+        {
+            AttVal *attr = GetAttrByName(node, "class");
+
+            /* map sequence of <p class="MsoListBullet"> to <ul>...</ul> */
+            if (attr && wstrcmp(attr->value, "MsoListBullet") == 0)
+            {
+                CoerceNode(lexer, node, tag_li);
+
+                if (!list || list->tag != tag_ul)
+                {
+                    list = InferredTag(lexer, "ul");
+                    InsertNodeBeforeElement(node, list);
+                }
+
+                PurgeAttributes(node);
+
+                if (node->content)
+                    CleanWord2000(lexer, node->content);
+
+                /* remove node and append to contents of list */
+                RemoveNode(node);
+                InsertNodeAtEnd(list, node);
+                node = list->next;
+            }
+            /* map sequence of <p class="Code"> to <pre>...</pre> */
+            else if (attr && wstrcmp(attr->value, "Code") == 0)
+            {
+                Node *br = NewLineNode(lexer);
+                NormalizeSpaces(lexer, node);
+
+                if (!list || list->tag != tag_pre)
+                {
+                    list = InferredTag(lexer, "pre");
+                    InsertNodeBeforeElement(node, list);
+                }
+
+                /* remove node and append to contents of list */
+                RemoveNode(node);
+                InsertNodeAtEnd(list, node);
+                StripSpan(lexer, node);
+                InsertNodeAtEnd(list, br);
+                node = list->next;
+            }
+            else
+                list = null;
+        }
+        else
+            list = null;
+
+        /* strip out style and class attributes */
+        if (node->type == StartTag || node->type == StartEndTag)
+            PurgeAttributes(node);
+
+        if (node->content)
+            CleanWord2000(lexer, node->content);
+
         node = node->next;
     }
 }
+
+Bool IsWord2000(Node *root)
+{
+    Node *html = FindHTML(root);
+
+    return (html && GetAttrByName(html, "xmlns:o"));
+}
