@@ -1,6 +1,6 @@
 /* html.h
 
-  (c) 1998 (W3C) MIT, INRIA, Keio University
+  (c) 1998-2000 (W3C) MIT, INRIA, Keio University
   See tidy.c for the copyright notice.
 */
 
@@ -19,6 +19,15 @@
 #define ISO2022     4
 #define MACROMAN    5
 
+typedef struct
+{
+    int encoding;
+    int state;     /* for ISO 2022 */
+    FILE *fp;
+} Out;
+
+void outc(uint c, Out *out);
+
 /* states for ISO 2022 
 
  A document in ISO-2022 based encoding uses some ESC sequences called 
@@ -57,9 +66,11 @@
 #define LEX_DOCTYPE     5
 #define LEX_PROCINSTR   6
 #define LEX_ENDCOMMENT  7
-#define LEX_SECTION     8
-#define LEX_ASP         9
-#define LEX_PHP         10
+#define LEX_CDATA       8
+#define LEX_SECTION     9
+#define LEX_ASP         10
+#define LEX_JSTE        11
+#define LEX_PHP         12
 
 /* content model shortcut encoding */
 
@@ -85,6 +96,7 @@
 #define CM_NO_INDENT    (1 << 18)
 #define CM_OBSOLETE     (1 << 19)
 #define CM_NEW          (1 << 20)
+#define CM_OMITST       (1 << 21)
 
 /*
  Linked list of class names and styles
@@ -121,6 +133,7 @@ typedef enum
     doctype_user
 } DocTypeMode;
 
+
 /*
  Attribute/Value linked list node
 */
@@ -149,25 +162,29 @@ typedef struct _attval AttVal;
 #define StartTag        5
 #define EndTag          6
 #define StartEndTag     7
-#define SectionTag      8
-#define AspTag          9
-#define PhpTag          10
+#define CDATATag        8
+#define SectionTag      9
+#define AspTag          10
+#define JsteTag         11
+#define PhpTag          12
 
 struct _node
 {
     struct _node *parent;
     struct _node *prev;
     struct _node *next;
+    struct _node *content;
     struct _node *last;
+    struct _attval *attributes;
+    char *element;          /* name (null for text nodes) */
     uint start;             /* start of span onto text array */
     uint end;               /* end of span onto text array */
     uint type;              /* TextNode, StartTag, EndTag etc. */
+    Bool closed;            /* true if closed by explicit end tag */
     Bool implicit;          /* true if inferred */
+    Bool linebreak;         /* true if followed by a line break */
     struct _tagdict *was;   /* old tag when it was changed */
     struct _tagdict *tag;   /* tag's dictionary definition */
-    char *element;          /* name (null for text nodes) */
-    struct _attval *attributes;
-    struct _node *content;
 };
 
 typedef struct _node Node;
@@ -277,6 +294,7 @@ struct _lexer
     Bool pushed;    /* true after token has been pushed back */
     Bool insertspace;   /* when space is moved after end tag */
     Bool excludeBlocks;  /* Netscape compatibility */
+    Bool exiled;    /* true if moved out of table */
     Bool isvoyager; /* true if xmlns attribute on html element */
     uint versions;  /* bit vector of HTML versions */
     int doctype;    /* version as given by doctype (if any) */
@@ -321,6 +339,7 @@ struct _attribute
     struct _attribute *next;
     char *name;
     Bool nowrap;
+    Bool literal;
     unsigned versions;
     AttrCheck *attrchk;
 };
@@ -360,6 +379,7 @@ typedef struct _tagdict Dict;
 #define IgnoreMarkup        3
 
 void FatalError(char *msg);
+void FileError(FILE *fp, const char *file);
 
 Node *GetToken(Lexer *lexer, uint mode);
 
@@ -380,11 +400,17 @@ Node *GetCDATA(Lexer *lexer, Node *container);
 /* use this to create node for inferred start tag */
 Node *InferredTag(Lexer *lexer, char *name);
 
+/* used to create line break in preformatted text
+   when cleaning the augean stables (Word2000) */
+Node *NewLineNode(Lexer *lexer);
+
 /* Parser calls this to create RootNode */
 Node *NewNode(void);
+AttVal *NewAttribute(void);
 
 void FreeAttrs(Node *node);
 void FreeAttribute(AttVal *av);
+void RemoveAttribute(Node *node, AttVal *attr);
 
 /* use this to free parse tree node and all its children */
 void FreeNode(Node *node);
@@ -408,11 +434,17 @@ Bool IsWhite(uint c);
 
 /* used to fixup doctype to match contents */
 Node *FindDocType(Node *root);
+Node *FindHTML(Node *root);
+Node *FindHEAD(Node *root);
+Bool AddGenerator(Lexer *lexer, Node *root);
 void DiscardDocType(Node *root);
 Bool FixDocType(Lexer *lexer, Node *node);
 char *HTMLVersionName(Lexer *lexer);
+int ApparentVersion(Lexer *lexer);
 Bool FixXMLPI(Lexer *lexer, Node *root);
 Bool SetXHTMLDocType(Lexer *lexer, Node *root);
+void FixId(Lexer *lexer, Node *node);
+Bool CheckDocTypeKeyWords(Lexer *lexer, Node *doctype);
 
 /* used to detect faulty attributes */
 Bool IsValidAttrName( char *attr);
@@ -421,8 +453,13 @@ Bool IsValidAttrName( char *attr);
 Node *ParseDocument(Lexer *lexer);
 Node *ParseXMLDocument(Lexer *lexer);
 Bool XMLPreserveWhiteSpace(Node *element);
+void CoerceNode(Lexer *lexer, Node *node, Dict *tag);
+Bool CheckNodeIntegrity(Node *node);
+Bool IsNewNode(Node *node);
 void RemoveNode(Node *node);
 Node *DiscardElement(Node *element);
+void InsertNodeAtStart(Node *element, Node *node);
+void InsertNodeAtEnd(Node *element, Node *node);
 void InsertNodeBeforeElement(Node *element, Node *node);
 void InsertNodeAfterElement(Node *element, Node *node);
 Bool IsJavaScript(Node *node);
@@ -432,9 +469,14 @@ void InitAttrs(void);
 void FreeAttrTable(void);
 Attribute *FindAttribute(AttVal *attval);
 AttVal *GetAttrByName(Node *node, char *name);
+void AddAttribute(Node *node, char *name, char *value);
+void CheckUniqueAttributes(Lexer *lexer, Node *node);
+void CheckAttributes(Lexer *lexer, Node *node);
 Attribute *CheckAttribute(Lexer *lexer, Node *node, AttVal *attval);
 Bool IsUrl(char *attrname);
 Bool IsScript(char *attrname);
+void DeclareLiteralAttrib(char *name);
+Bool IsLiteralAttribute(char *attrname);
 
 /* istack.c */
 void PushInline(Lexer *lexer, Node *node);
@@ -443,14 +485,20 @@ Bool IsPushed(Lexer *lexer, Node *node);
 int InlineDup(Lexer *lexer, Node *node);
 Node *InsertedToken(Lexer *lexer);
 AttVal *DupAttrs(AttVal *attrs);
+void DeferDup(Lexer *lexer);
 void InsertNode(Node *element, Node *node);
 
 /* clean.c */
 void FreeStyles(Lexer *lexer);
+void AddClass(Node *node, char *classname);
 void CleanTree(Lexer *lexer, Node *node);
+void NestedEmphasis(Node *node);
 void EmFromI(Node *node);
-void CleanWord2000(Node *node);
+void CleanWord2000(Lexer *lexer, Node *node);
 void DropSections(Lexer *lexer, Node *node);
+void List2BQ(Node *node);
+void BQ2Div(Node *node);
+Bool IsWord2000(Node *root);
 
 /* entities.c */
 void InitEntities(void);
@@ -462,23 +510,31 @@ char *EntityName(uint n);
 void DefineEmptyTag(char *name);
 void DefineInlineTag(char *name);
 void DefineBlockTag(char *name);
+void DefinePreTag(char *name);
 
 Bool FindTag(Node *node);
 void InitTags(void);
 void FreeTags(void);
+Parser *FindParser(Node *node);
 int HTMLVersion(Lexer *lexer);
 
 /* localize.c -- used for all message text */
-
+void ShowVersion(FILE *fp);
+void ReadingFromStdin(void);
+void ReportUnknownOption(char *option);
+void ReportBadArgument(char *option);
 void NeedsAuthorIntervention(FILE *errout);
 void MissingBody(FILE *errout);
 void ReportNumberOfSlides(FILE *errout, int count);
 void GeneralInfo(FILE *errout);
 void HelloMessage(FILE *errout, char *date, char *filename);
-void ReportVersion(FILE *errout, char *filename, char *vers);
+void ReportVersion(FILE *errout, Lexer *lexer,
+                   char *filename, Node *doctype);
 void ReportNumWarnings(FILE *errout, Lexer *lexer);
 
 /* pprint.c */
+uint GetUTF8(unsigned char *str, uint *ch);
+char *PutUTF8(char *buf, uint c);
 void FreePrintBuf(void);
 void PPrintTree(Out *out, uint mode, uint indent,
                     Lexer *lexer, Node *node);
@@ -486,8 +542,9 @@ void PPrintXMLTree(Out *fout, uint mode, uint indent,
                     Lexer *lexer, Node *node);
 void PFlushLine(Out *out, uint indent);
 void PCondFlushLine(Out *out, uint indent);
-void PPrintAsp(Out *fout, uint indent, Lexer *lexer, Node *node);
-void PPrintPhp(Out *fout, uint indent, Lexer *lexer, Node *node);
+void AddTransitionEffect(Lexer *lexer, Node *root, int effect, float duration);
+Node *FindHead(Node *root);
+Node *FindBody(Node *root);
 
 /* tidy.c */
 #define EndOfStream EOF
@@ -495,13 +552,14 @@ void PPrintPhp(Out *fout, uint indent, Lexer *lexer, Node *node);
 void *MemAlloc(uint size);
 void *MemRealloc(void *mem, uint newsize);
 void MemFree(void *mem);
-int ReadChar(StreamIn *in);
-void UngetChar(int c, StreamIn *in);
 
 /* string functions */
+uint ToLower(uint c);
 char *wstrdup(char *str);
 char *wstrndup(char *str, int len);
 void wstrncpy(char *s1, char *s2, int size);
+void wstrcat(char *s1, char *s2);
+void wstrcpy(char *s1, char *s2);
 int wstrcmp(char *s1, char *s2);
 int wstrcasecmp(char *s1, char *s2);    
 int wstrncmp(char *s1, char *s2, int n);
@@ -510,6 +568,8 @@ int wstrlen(char *str);
 Bool wsubstr(char *s1, char *s2);
 void ClearMemory(void *, uint size);
 
+void tidy_out(FILE *fp, const char* msg, ...);
+
 #define uprintf fprintf
 
 /* error codes for entities */
@@ -523,31 +583,36 @@ void ClearMemory(void *, uint size);
 #define MISSING_ENDTAG_FOR      1
 #define MISSING_ENDTAG_BEFORE   2
 #define DISCARDING_UNEXPECTED   3
-#define NON_MATCHING_ENDTAG     4
-#define TAG_NOT_ALLOWED_IN      5
-#define MISSING_STARTTAG        6
-#define UNEXPECTED_ENDTAG       7
-#define USING_BR_INPLACE_OF     8
-#define INSERTING_TAG           9
-#define SUSPECTED_MISSING_QUOTE 10
-#define MISSING_TITLE_ELEMENT   11
-#define DUPLICATE_FRAMESET      12
-#define CANT_BE_NESTED          13
-#define OBSOLETE_ELEMENT        14
-#define PROPRIETARY_ELEMENT     15
-#define UNKNOWN_ELEMENT         16
-#define TRIM_EMPTY_ELEMENT      17
-#define FORCED_END_ANCHOR       18
-#define ILLEGAL_NESTING         19
-#define NOFRAMES_CONTENT        20
-#define CONTENT_AFTER_BODY      21
-#define INCONSISTENT_VERSION    22
-#define BAD_COMMENT             23
-#define BAD_CDATA_CONTENT       24
-#define INCONSISTENT_NAMESPACE  25
-#define DOCTYPE_AFTER_TAGS      26
-#define MALFORMED_DOCTYPE       27
-#define UNEXPECTED_END_OF_FILE  28
+#define NESTED_EMPHASIS         4
+#define NON_MATCHING_ENDTAG     5
+#define TAG_NOT_ALLOWED_IN      6
+#define MISSING_STARTTAG        7
+#define UNEXPECTED_ENDTAG       8
+#define USING_BR_INPLACE_OF     9
+#define INSERTING_TAG           10
+#define SUSPECTED_MISSING_QUOTE 11
+#define MISSING_TITLE_ELEMENT   12
+#define DUPLICATE_FRAMESET      13
+#define CANT_BE_NESTED          14
+#define OBSOLETE_ELEMENT        15
+#define PROPRIETARY_ELEMENT     16
+#define UNKNOWN_ELEMENT         17
+#define TRIM_EMPTY_ELEMENT      18
+#define COERCE_TO_ENDTAG        19
+#define ILLEGAL_NESTING         20
+#define NOFRAMES_CONTENT        21
+#define CONTENT_AFTER_BODY      22
+#define INCONSISTENT_VERSION    23
+#define MALFORMED_COMMENT       24
+#define BAD_COMMENT_CHARS       25
+#define BAD_XML_COMMENT         26
+#define BAD_CDATA_CONTENT       27
+#define INCONSISTENT_NAMESPACE  28
+#define DOCTYPE_AFTER_TAGS      29
+#define MALFORMED_DOCTYPE       30
+#define UNEXPECTED_END_OF_FILE  31
+#define DTYPE_NOT_UPPER_CASE    32
+#define TOO_MANY_ELEMENTS       33
 
 /* error codes used for attribute messages */
 
@@ -557,9 +622,39 @@ void ClearMemory(void *, uint size);
 #define BAD_ATTRIBUTE_VALUE     4
 #define UNEXPECTED_GT           5
 #define PROPRIETARY_ATTR_VALUE  6
-#define MISSING_IMAGEMAP        7
-#define XML_ATTRIBUTE_VALUE     8
-#define UNEXPECTED_QUOTEMARK    9
+#define REPEATED_ATTRIBUTE      7
+#define MISSING_IMAGEMAP        8
+#define XML_ATTRIBUTE_VALUE     9
+#define UNEXPECTED_QUOTEMARK    10
+#define ID_NAME_MISMATCH        11
+
+/* page transition effects */
+
+#define EFFECT_BLEND               -1
+#define EFFECT_BOX_IN               0
+#define EFFECT_BOX_OUT              1
+#define EFFECT_CIRCLE_IN            2
+#define EFFECT_CIRCLE_OUT           3
+#define EFFECT_WIPE_UP              4
+#define EFFECT_WIPE_DOWN            5
+#define EFFECT_WIPE_RIGHT           6
+#define EFFECT_WIPE_LEFT            7
+#define EFFECT_VERT_BLINDS          8
+#define EFFECT_HORZ_BLINDS          9
+#define EFFECT_CHK_ACROSS          10
+#define EFFECT_CHK_DOWN            11
+#define EFFECT_RND_DISSOLVE        12
+#define EFFECT_SPLIT_VIRT_IN       13
+#define EFFECT_SPLIT_VIRT_OUT      14
+#define EFFECT_SPLIT_HORZ_IN       15
+#define EFFECT_SPLIT_HORZ_OUT      16
+#define EFFECT_STRIPS_LEFT_DOWN    17
+#define EFFECT_STRIPS_LEFT_UP      18
+#define EFFECT_STRIPS_RIGHT_DOWN   19
+#define EFFECT_STRIPS_RIGHT_UP     20
+#define EFFECT_RND_BARS_HORZ       21
+#define EFFECT_RND_BARS_VERT       22
+#define EFFECT_RANDOM              23
 
 /* accessibility flaws */
 
@@ -576,6 +671,7 @@ void ClearMemory(void *, uint size);
 #define USING_LAYER             2
 #define USING_NOBR              4
 #define USING_FONT              8
+#define USING_BODY              16
 
 /* character encoding errors */
 #define WINDOWS_CHARS           1
@@ -606,7 +702,7 @@ void CreateSlides(Lexer *lexer, Node *root);
 void InitConfig(void);
 void FreeConfig(void);
 void ParseConfigFile(char *file);
-void ParseConfig(char *option, char *parameter);
+Bool ParseConfig(char *option, char *parameter);
 void AdjustConfig(void);
 
 extern uint spaces;         /* default indentation */
@@ -622,6 +718,7 @@ extern char *errfile;       /* file name to write errors to */
 extern Bool writeback;      /* if true then output tidied markup */
 extern Bool KeepFileTimes;  /* if true keeps last modified time */
 
+extern Bool TidyMark;       /* add meta element indicating tidied doc */
 extern Bool OnlyErrors;     /* if true normal output is suppressed */
 extern Bool ShowWarnings;   /* errors are always shown */
 extern Bool Quiet;
@@ -633,26 +730,34 @@ extern Bool XmlOut;
 extern Bool xHTML;
 extern Bool XmlPi;      /* add <?xml?> */
 extern Bool XmlPIs;     /* assume PIs end with ?> as per XML */
+extern Bool XmlSpace;
 extern Bool RawOut;
 extern Bool UpperCaseTags;
 extern Bool UpperCaseAttrs;
 extern Bool MakeClean;
 extern Bool LogicalEmphasis;
+extern Bool DropEmptyParas;
+extern Bool FixComments;
 extern Bool DropFontTags;
-extern Bool EncloseText;
+extern Bool EncloseBodyText;
+extern Bool EncloseBlockText;
 extern Bool BurstSlides;
 extern Bool BreakBeforeBR;
 extern Bool NumEntities;
 extern Bool QuoteMarks;
 extern Bool QuoteNbsp;
 extern Bool QuoteAmpersand;
+extern Bool WrapAttVals;
 extern Bool WrapScriptlets;
 extern Bool WrapSection;
 extern Bool WrapAsp;
+extern Bool WrapJste;
 extern Bool WrapPhp;
 extern Bool FixBackslash;
 extern Bool IndentAttributes;
 extern Bool Word2000;
+extern Bool Emacs;  /* sasdjb 01May00 GNU Emacs error output format */
+extern Bool LiteralAttribs;
 
 /* Parser methods for tags */
 
@@ -681,11 +786,17 @@ Parser ParseMap;
 
 /* Attribute checking methods */
 
+CheckAttribs CheckHR;
 CheckAttribs CheckIMG;
+CheckAttribs CheckAnchor;
+CheckAttribs CheckLINK;
+CheckAttribs CheckMap;
 CheckAttribs CheckAREA;
 CheckAttribs CheckTABLE;
+CheckAttribs CheckTableCell;
 CheckAttribs CheckCaption;
 CheckAttribs CheckSCRIPT;
+CheckAttribs CheckSTYLE;
 CheckAttribs CheckHTML;
 
 /* used to control printing of null attributes */
@@ -695,8 +806,10 @@ extern Dict *tag_html;
 extern Dict *tag_head;
 extern Dict *tag_body;
 extern Dict *tag_frameset;
+extern Dict *tag_frame;
 extern Dict *tag_noframes;
 extern Dict *tag_title;
+extern Dict *tag_base;
 extern Dict *tag_hr;
 extern Dict *tag_meta;
 extern Dict *tag_pre;
@@ -722,9 +835,12 @@ extern Dict *tag_b;
 extern Dict *tag_i;
 extern Dict *tag_strong;
 extern Dict *tag_em;
+extern Dict *tag_big;
+extern Dict *tag_small;
 extern Dict *tag_param;
 extern Dict *tag_option;
 extern Dict *tag_optgroup;
+extern Dict *tag_img;
 extern Dict *tag_map;
 extern Dict *tag_area;
 extern Dict *tag_nobr;
@@ -739,6 +855,9 @@ extern Dict *tag_noscript;
 extern Dict *tag_table;
 extern Dict *tag_caption;
 extern Dict *tag_form;
+extern Dict *tag_textarea;
 extern Dict *tag_blockquote;
+extern Dict *tag_applet;
+extern Dict *tag_object;
 extern Dict *tag_div;
 extern Dict *tag_span;
