@@ -1,10 +1,10 @@
 /*
   tidy.c - HTML parser and pretty printer
 
-  Copyright (c) 1998 World Wide Web Consortium (Massachusetts
-  Institute of Technology, Institut National de Recherche en
-  Informatique et en Automatique, Keio University). All Rights
-  Reserved.
+  Copyright (c) 1998-2000 World Wide Web Consortium
+  (Massachusetts Institute of Technology, Institut National de
+  Recherche en Informatique et en Automatique, Keio University).
+  All Rights Reserved.
 
   Contributing Author(s):
 
@@ -58,9 +58,19 @@ Node       *debug_element = null;
 Lexer      *debug_lexer = null;
 uint       totalerrors = 0;
 uint       totalwarnings = 0;
+uint       optionerrors = 0;
 
-jmp_buf error_exit;  /* Address for long jump to jump to */
 FILE *errout;  /* set to stderr or stdout */
+FILE *input;
+
+/* Mapping for Windows Western character set (128-159) to Unicode */
+int Win2Unicode[32] =
+{
+    0x20AC, 0x0000, 0x201A, 0x0192, 0x201E, 0x2026, 0x2020, 0x2021,
+    0x02C6, 0x2030, 0x0160, 0x2039, 0x0152, 0x0000, 0x017D, 0x0000,
+    0x0000, 0x2018, 0x2019, 0x201C, 0x201D, 0x2022, 0x2013, 0x2014,
+    0x02DC, 0x2122, 0x0161, 0x203A, 0x0153, 0x0000, 0x017E, 0x0178
+};
 
 /*
 John Love-Jensen contributed this table for mapping MacRoman
@@ -123,7 +133,12 @@ void FatalError(char *msg)
 {
     fprintf(stderr, "Fatal error: %s\n", msg);
     DeInitTidy();
-    longjmp(error_exit, -1);
+
+    if (input && input != stdin)
+        fclose(input);
+
+    /* 2 signifies a serious error */
+    exit(2);
 }
 
 void *MemAlloc(uint size)
@@ -182,7 +197,7 @@ StreamIn *OpenInput(FILE *fp)
 }
 
 /* read char from stream */
-int ReadCharFromStream(StreamIn *in)
+static int ReadCharFromStream(StreamIn *in)
 {
     uint n, c, i, count;
 
@@ -375,45 +390,9 @@ int ReadChar(StreamIn *in)
         {
             ReportEncodingError(in->lexer, WINDOWS_CHARS, c);
 
-            if (c == 150)
-                c = 8211;  /* en dash */
-            else if (c == 151)
-                c = 8212;  /* em dash */
-            else if (c == 138)
-                c = 352;  /* latin capital letter S with caron */
-            else if (c == 154)
-                c = 353;  /* latin small letter s with caron */
-            else if (c == 159)
-                c = 376;  /* latin capital letter Y with diaeresis */
-            else if (c == 140)
-                c = 338;  /* latin capital ligature OE */
-            else if (c == 156)
-                c = 339;  /* latin capital ligature OE */
-            else if (c == 153)
-                c = 8482;  /* TM */
-            else if (c == 134)
-                c = 8224;  /* dagger */
-            else if (c == 135)
-                c = 8225;  /* double dagger */
-            else if (c == 137)
-                c = 8240;  /* per mille sign */
-            else if (c == 130)
-                c = 8218;  /* single low quotation mark */
-            else if (c == 132)
-                c = 8222;  /* double low quotation mark */
-            else if (c == 145)
-                c = 8216;  /* single left quotation mark */
-            else if (c == 146)
-                c = 8217;  /* single right quotation mark */
-            else if (c == 147)
-                c = 8220;  /* double left quotation mark */
-            else if (c == 148)
-                c = 8221;  /* double right quotation mark */
-            else if (c == 139)
-                c = 8249;  /* single left-pointing angle quotation mark */
-            else if (c == 155)
-                c = 8250;  /* single right-pointing angle quotation mark */
-            else
+            c = Win2Unicode[c - 128];
+
+            if (c == 0)
                 continue;
         }
 
@@ -489,6 +468,14 @@ void wstrcpy(char *s1, char *s2)
     while ((*s1++ = *s2++));
 }
 
+void wstrcat(char *s1, char *s2)
+{
+    while (*s1)
+        ++s1;
+
+    while ((*s1++ = *s2++));
+}
+
 /* exactly same as strcmp */
 int wstrcmp(char *s1, char *s2)    
 {
@@ -693,6 +680,7 @@ void InitTidy(void)
     XmlTags = XmlOut = HideEndTags = UpperCaseTags =
     MakeClean = writeback = OnlyErrors = no;
 
+    input = null;
     errfile = null;
     errout = stderr;
 
@@ -717,11 +705,9 @@ void DeInitTidy(void)
 int main(int argc, char **argv)
 {
     char *file, *prog;
-    FILE *fp = null;
-    Node *document;
+    Node *document, *doctype;
     Lexer *lexer;
     char *s, c, *arg, *current_errorfile = "stderr";
-    int jmpret;
     Out out;   /* normal output stream */
 
 #if PRESERVEFILETIMES
@@ -729,31 +715,18 @@ int main(int argc, char **argv)
     struct stat sbuf;
 #endif
     Bool haveFileTimes;
-    /*
-      set up for long jump back to here on severe errors
-    */
-    jmpret = setjmp(error_exit);
-
-    /*
-      return on a severe error after long jump
-    */
-
-    if (jmpret != 0)
-    {
-        /* ensure input is closed */
-        if (fp && fp != stdin)
-            fclose(fp);
-
-        /* 2 signifies a serious error */
-        return 2;
-    }
 
     InitTidy();
 
     /* look for env var "HTML_TIDY" */
+    /* then for ~/.tidyrc (on Unix) */
 
     if ((file = getenv("HTML_TIDY")))
         ParseConfigFile(file);
+#ifdef SUPPORT_GETPWNAM
+    else
+        ParseConfigFile("~/.tidyrc");
+#endif /* SUPPORT_GETPWNAM */
 
     /* read command line */
 
@@ -765,15 +738,15 @@ int main(int argc, char **argv)
         {
             /* support -foo and --foo */
             arg = argv[1] + 1;
-
+#if 0
             if (arg[0] == '-')
                 ++arg;
-
+#endif
             if (strcmp(arg, "indent") == 0)
                 IndentContent = yes;
             else if (strcmp(arg, "xml") == 0)
                 XmlTags = yes;
-            else if (strcmp(arg, "asxml") == 0)
+            else if (strcmp(arg, "asxml") == 0 || strcmp(arg, "asxhtml") == 0)
                 xHTML = yes;
             else if (strcmp(arg, "indent") == 0)
             {
@@ -850,11 +823,23 @@ int main(int argc, char **argv)
                     ++argv;
                 }
             }
+            else if (strcmp(argv[1], "-version") == 0 ||
+                        strcmp(argv[1], "--version") == 0 ||
+                        strcmp(argv[1], "-v") == 0)
+            {
+                ShowVersion(errout);
+                /* called to free hash tables etc. */
+                DeInitTidy();
+                return 0;
+
+            }
             else if(strncmp(argv[1],"--",2)==0)
             {
-                ParseConfig(argv[1]+2, argv[2]);
-                ++argv;
-                --argc;
+                if (ParseConfig(argv[1]+2, argv[2]))
+                {
+                    ++argv;
+                    --argc;
+                }
             }
             else
             {
@@ -897,6 +882,8 @@ int main(int argc, char **argv)
         /* user specified error file */
         if (errfile)
         {
+            FILE *fp;
+
             /* is it same as the currently opened file? */
             if (wstrcmp(errfile, current_errorfile) != 0)
             {
@@ -926,11 +913,11 @@ int main(int argc, char **argv)
         if (argc > 1)
         {
             file = argv[1];
-            fp = fopen(file, "r");
+            input = fopen(file, "r");
 
 #if PRESERVEFILETIMES
             /* get last modified time */
-            if (KeepFileTimes && fp && fstat(fileno(fp), &sbuf) != -1)
+            if (KeepFileTimes && input && fstat(fileno(input), &sbuf) != -1)
             {
                 filetimes.actime = sbuf.st_atime;
                 filetimes.modtime = sbuf.st_mtime;
@@ -940,13 +927,13 @@ int main(int argc, char **argv)
         }
         else
         {
-            fp = stdin;
+            input = stdin;
             file = "stdin";
         }
 
-        if (fp != null)
+        if (input != null)
         {
-            lexer = NewLexer(OpenInput(fp));
+            lexer = NewLexer(OpenInput(input));
             lexer->errout = errout;
 
             /*
@@ -968,29 +955,53 @@ int main(int argc, char **argv)
 
                 document = ParseDocument(lexer);
 
+                if (!CheckNodeIntegrity(document))
+                {
+                    fprintf(stderr, "\nPanic - tree has lost its integrity\n");
+                    exit(1);
+                }
+
+                /* simplifies <b><b> ... </b> ...</b> etc. */
+                NestedEmphasis(document);
+
+                /* cleans up <dir>indented text</dir> etc. */
+                List2BQ(document);
+                BQ2Div(document);
+
                 /* replaces i by em and b by strong */
                 if (LogicalEmphasis)
                     EmFromI(document);
 
-                if (Word2000)
+                if (Word2000 && IsWord2000(document))
                 {
                     /* prune Word2000's <![if ...]> ... <![endif]> */
                     DropSections(lexer, document);
 
                     /* drop style & class attributes and empty p, span elements */
-                    CleanWord2000(document);
+                    CleanWord2000(lexer, document);
                 }
 
                 /* replaces presentational markup by style rules */
-                if (MakeClean)
+                if (MakeClean || DropFontTags)
                     CleanTree(lexer, document);
 
+                if (!CheckNodeIntegrity(document))
+                {
+                    fprintf(stderr, "\nPanic - tree has lost its integrity\n");
+                    exit(1);
+                }
+
+                doctype = FindDocType(document);
+
                 if (document->content)
                 {
                     if (xHTML)
                         SetXHTMLDocType(lexer, document);
                     else
                         FixDocType(lexer, document);
+
+                    if (TidyMark)
+                        AddGenerator(lexer, document);
                 }
 
                 /* ensure presence of initial <?XML version="1.0"?> */
@@ -1002,14 +1013,14 @@ int main(int argc, char **argv)
 
                 if (!Quiet && document->content)
                 {
-                    ReportVersion(errout, file, HTMLVersionName(lexer));
+                    ReportVersion(errout, lexer, file, doctype);
                     ReportNumWarnings(errout, lexer);
                 }
             }
 
-            if (fp != stdin)
+            if (input != stdin)
             {
-                fclose(fp);
+                fclose(input);
             }
 
             MemFree(lexer->in);
@@ -1024,20 +1035,20 @@ int main(int argc, char **argv)
             {
                 if (BurstSlides)
                 {
-                    Node *body, *doctype;
+                    Node *body, *this_doctype;
 
                     /*
                        remove doctype to avoid potential clash with
                        markup introduced when bursting into slides
                     */
                     /* discard the document type */
-                    doctype = FindDocType(document);
+                    this_doctype = FindDocType(document);
 
-                    if (doctype)
-                        DiscardElement(doctype);
+                    if (this_doctype)
+                        DiscardElement(this_doctype);
 
                     /* slides use transitional features */
-                    lexer->versions |= VERS_LOOSE;
+                    lexer->versions |= VERS_HTML40_LOOSE;
 
                     /* and patch up doctype to match */
                     if (xHTML)
@@ -1057,9 +1068,9 @@ int main(int argc, char **argv)
                     else
                         MissingBody(errout);
                 }
-                else if (writeback && (fp = fopen(file, "w")))
+                else if (writeback && (input = fopen(file, "w")))
                 {
-                    out.fp = fp;
+                    out.fp = input;
 
                     if (XmlTags)
                         PPrintXMLTree(&out, null, 0, lexer, document);
@@ -1071,9 +1082,9 @@ int main(int argc, char **argv)
 #if PRESERVEFILETIMES
                     /* set file last accessed/modified times to original values */
                     if (haveFileTimes)
-                        futime(fileno(fp), &filetimes);
+                        futime(fileno(input), &filetimes);
 #endif
-                    fclose(fp);
+                    fclose(input);
                 }
                 else
                 {
