@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -40,6 +40,10 @@
 #include "eperl_perl5.h"
 #include "eperl_proto.h"
 
+#define _EPERL_VERSION_C_AS_HEADER_
+#include "eperl_version.c"
+#undef  _EPERL_VERSION_C_AS_HEADER_
+
 int mode = MODE_UNKNOWN;
 
 char *allowed_file_ext[]   = LIST_OF_ALLOWED_FILE_EXT;
@@ -80,7 +84,7 @@ void PrintError(int mode, char *scripturl, char *scriptfile, char *logfile, char
         printf("<td><img src=\"%s/logo.gif\" alt=\"Embedded Perl 5 Language\"></td>\n", cp);
         printf("</tr>\n");
         printf("<tr>\n");
-        printf("<td align=right><b>Version %s</b></td>\n", ePerl_Version);
+        printf("<td align=right><b>Version %s</b></td>\n", eperl_version.v_short);
         printf("</tr>\n");
         printf("</table>\n");
         printf("<p>\n");
@@ -133,9 +137,9 @@ void PrintError(int mode, char *scripturl, char *scriptfile, char *logfile, char
 
 void give_version(void)
 {
-    fprintf(stdout, "%s\n", ePerl_Hello);
+    fprintf(stdout, "%s\n", eperl_version.v_tex);
     fprintf(stdout, "\n");
-    fprintf(stdout, "Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>\n");
+    fprintf(stdout, "Copyright (c) 1996-2000 Ralf S. Engelschall <rse@engelschall.com>\n");
     fprintf(stdout, "\n");
     fprintf(stdout, "This program is distributed in the hope that it will be useful,\n");
     fprintf(stdout, "but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
@@ -333,7 +337,7 @@ int main(int argc, char **argv, char **env)
     struct passwd *pw;
     struct passwd *pw2;
     struct group *gr;
-    int uid, gid;
+    int uid = 0, gid = 0;
     int keepcwd = FALSE;
     int c;
     char *cpScript = NULL;
@@ -966,7 +970,7 @@ int main(int argc, char **argv, char **env)
         env = mysetenv(env, "SCRIPT_SRC_OWNER", "%s", pw->pw_name);
     else
         env = mysetenv(env, "SCRIPT_SRC_OWNER", "unknown-uid-%d", st.st_uid);
-    env = mysetenv(env, "VERSION_INTERPRETER", "%s", ePerl_WebID);
+    env = mysetenv(env, "VERSION_INTERPRETER", "%s", eperl_version.v_web);
     env = mysetenv(env, "VERSION_LANGUAGE", "Perl/%s", AC_perl_vers);
 
     /* optionally run the ePerl preprocessor */
@@ -1047,14 +1051,18 @@ int main(int argc, char **argv, char **env)
     }
     IO_redirect_stderr(er);
 
+    /*  initialise the Perl Locale environment  */
+#if AC_perl_vnum < 500400
+    perl_init_i18nl14n(1); /* Perl 5.003 or lower */
+#else
+    perl_init_i18nl10n(1); /* Perl 5.004 or higher */
+#endif
+
     /*  now allocate the Perl interpreter  */
     my_perl = perl_alloc();   
     perl_construct(my_perl); 
     /* perl_destruct_level = 1; */
 
-    /*  initialise the Perl Locale environment  */
-    perl_init_i18nl10n(1); /* Macro defined in eperl_perl5.h */
-
     /*  create command line...  */
     myargc = 0;
     /*  - program name and possible -T -w options */
