@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -286,7 +286,7 @@ char *ePerl_Cfwrite(char *cpBuf, int nBuf, int cNum, char *cpOut)
 **
 */
 
-char *strnchr(char *buf, char chr, int n)
+char *ep_strnchr(char *buf, char chr, int n)
 {
     char *cp;
     char *cpe;
@@ -298,7 +298,7 @@ char *strnchr(char *buf, char chr, int n)
     return NULL;
 }
 
-char *strnstr(char *buf, char *str, int n)
+char *ep_strnstr(char *buf, char *str, int n)
 {
     char *cp;
     char *cpe;
@@ -312,7 +312,7 @@ char *strnstr(char *buf, char *str, int n)
     return NULL;
 }
 
-char *strncasestr(char *buf, char *str, int n)
+char *ep_strncasestr(char *buf, char *str, int n)
 {
     char *cp;
     char *cpe;
@@ -326,7 +326,7 @@ char *strncasestr(char *buf, char *str, int n)
     return NULL;
 }
 
-char *strndup(char *buf, int n)
+char *ep_strndup(char *buf, int n)
 {
     char *cp;
 
@@ -378,9 +378,9 @@ char *ePerl_Bristled2Plain(char *cpBuf)
     while (cps < cpEND) {
 
         if (ePerl_case_sensitive_delimiters)
-            cpe = strnstr(cps, ePerl_begin_delimiter, cpEND-cps);
+            cpe = ep_strnstr(cps, ePerl_begin_delimiter, cpEND-cps);
         else
-            cpe = strncasestr(cps, ePerl_begin_delimiter, cpEND-cps);
+            cpe = ep_strncasestr(cps, ePerl_begin_delimiter, cpEND-cps);
         if (cpe == NULL) {
 
             /* there are no more ePerl blocks, so
@@ -390,7 +390,7 @@ char *ePerl_Bristled2Plain(char *cpBuf)
             if (cps < cpEND) {
                 cps2 = cps;
                 /* first, do all complete lines */
-                while (cps2 < cpEND && (cpe2 = strnchr(cps2, '\n', cpEND-cps2)) != NULL) {
+                while (cps2 < cpEND && (cpe2 = ep_strnchr(cps2, '\n', cpEND-cps2)) != NULL) {
                     if (ePerl_line_continuation && cps < cpe2 && *(cpe2-1) == '\\') {
                         if (cpe2-1-cps2 > 0) {
                             cpOut = ePerl_fprintf(cpOut, "print \"");
@@ -425,7 +425,7 @@ char *ePerl_Bristled2Plain(char *cpBuf)
                up to the begin of the ePerl block as print statements */
             if (cps < cpe) {
                 cps2 = cps;
-                while ((cpe2 = strnchr(cps2, '\n', cpe-cps2)) != NULL) {
+                while ((cpe2 = ep_strnchr(cps2, '\n', cpe-cps2)) != NULL) {
                     if (ePerl_line_continuation && cps < cpe2 && *(cpe2-1) == '\\') {
                         if (cpe2-1-cps2 > 0) {
                             cpOut = ePerl_fprintf(cpOut, "print \"");
@@ -476,9 +476,9 @@ char *ePerl_Bristled2Plain(char *cpBuf)
 
             /* move forward to end of ePerl block. */
             if (ePerl_case_sensitive_delimiters)
-                cpe = strnstr(cpe, ePerl_end_delimiter, cpEND-cpe);
+                cpe = ep_strnstr(cpe, ePerl_end_delimiter, cpEND-cpe);
             else
-                cpe = strncasestr(cpe, ePerl_end_delimiter, cpEND-cpe);
+                cpe = ep_strncasestr(cpe, ePerl_end_delimiter, cpEND-cpe);
             if (cpe == NULL) {
                 ePerl_SetError("Missing end delimiter");
                 CU(NULL);
