@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -66,8 +66,12 @@ void Perl5_XSInit(void)
 */
 void Perl5_ForceUnbufferedStdout(void)
 {
+#if AC_perl_vnum < 500476
+    IoFLAGS(GvIOp(defoutgv)) |= IOf_FLUSH; /* $|=1 */
+#else
     dTHR;
     IoFLAGS(GvIOp(PL_defoutgv)) |= IOf_FLUSH; /* $|=1 */
+#endif
     return;
 }
 
@@ -94,10 +98,17 @@ char **Perl5_SetEnvVar(char **env, char *str)
 */
 void Perl5_SetScalar(char *pname, char *vname, char *vvalue)
 {
+#if AC_perl_vnum >= 500476
     dTHR;
+#endif
     ENTER;
+#if AC_perl_vnum >= 500600
     save_hptr(&PL_curstash); 
     PL_curstash = gv_stashpv(pname, TRUE);
+#else
+    save_hptr(&curstash); 
+    curstash = gv_stashpv(pname, TRUE);
+#endif
     sv_setpv(perl_get_sv(vname, TRUE), vvalue);
     LEAVE;
     return;
