@@ -7,7 +7,7 @@ open(OUT, ">wml_tags.pod");
 open(IN, "<wml_tags.pod.in");
 while (<IN>) {
     if (m|^%%CORE%%|) {
-        open(TMP, "<wml_tags.L.core");
+        open(TMP, "<wml_tags.L.main");
         @L = ();
         while (<TMP>) { 
             next if (m|^\s*$|);
